/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator module.
 * It is responsible for managing user interface.
 *
 * @module app.ui.creator
 * @requires {@link app.common.pages}
 * @requires {@link app.common.validation}
 * @requires {@link app.common.events}
 * @requires {@link app.model.option}
 * @requires {@link app.model.notification}
 * @requires {@link app.ui.templates}
 * @requires {@link app.ui.creator.option}
 * @namespace app.ui.creator
 * @memberof app.ui
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreator(app) {
    'use strict';

    /**
     * Common pages module reference.
     *
     * @memberof app.ui.creator
     * @private
     * @type {object}
     */
    var commonPages = app.common.pages,

        /**
         * Common validation module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        commonValidation = app.common.validation,

        /**
         * Common events module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        commonEvents = app.common.events,

        /**
         * UI templates module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        uiTemplates = app.ui.templates,

        /**
         * Model option module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        modelOption = app.model.option,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * UI creator option module reference.
         *
         * @memberof app.ui.creator
         * @private
         * @type {object}
         */
        uiCreatorOption = app.ui.creator.option,

        /**
         * Animated CSS class.
         *
         * @memberof app.ui.creator
         * @private
         * @const {string}
         */
        ANIMATED_CLASS = 'animated',

        /**
         * Active CSS class.
         *
         * @memberof app.ui.creator
         * @private
         * @const {string}
         */
        ACTIVE_CLASS = 'active',

        /**
         * Keyboard on CSS class.
         *
         * @memberof app.ui.creator
         * @private
         * @const {string}
         */
        KEYBOARD_ON_CLASS = 'keyboard-on',

        /**
         * Switching threshold factor CSS class.
         *
         * @memberof app.ui.creator
         * @private
         * @const {number}
         */
        SWITCHING_THRESHOLD_FACTOR_CLASS = 7,

        /**
         * Page element.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        page = null,

        /**
         * Back button.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        backBtn = null,

        /**
         * Code button.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        codeBtn = null,

        /**
         * Send button.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        sendBtn = null,

        /**
         * Element that contains animated option elements.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        optionsContainer = null,

        /**
         * Element that contains indicator elements.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        indicatorContainer = null,

        /**
         * Element that contains details related to the chosen option.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Notification type.
         *
         * @memberof app.ui.creator
         * @private
         * @type {string}
         */
        notificationType = '',

        /**
         * Array that stores references to the option HTML elements.
         *
         * @memberof app.ui.creator
         * @private
         * @type {HTMLElements[]}
         */
        options = null,

        /**
         * Id of the active option.
         *
         * @memberof app.ui.creator
         * @private
         * @type {number}
         */
        activeOptionId = 0,

        /**
         * Start position of the options element before move.
         *
         * @memberof app.ui.creator
         * @private
         * @type {number}
         */
        optionsXStart = 0,

        /**
         * The value equal to the sum of movement vectors for each moving step
         * of the options element.
         *
         * @memberof app.ui.creator
         * @private
         * @type {number}
         */
        optionsXVectorSum = 0,

        /**
         * Previous difference between the cursor position
         * saved on touch start event on the options element,
         * and saved while moving step.
         *
         * @memberof app.ui.creator
         * @private
         * @type {number}
         */
        optionsXDeltaBuffer = 0,

        /**
         * Page width.
         *
         * @memberof app.ui.creator
         * @private
         * @type {number}
         */
        pageWidth = 0,

        /**
         * Indicates whether page content should not be reset
         * after pagehide event.
         *
         * @memberof app.ui.creator
         * @private
         * @type {boolean}
         */
        doNotResetPageContent = false;

    /**
     * Clears option details content.
     *
     * @memberof app.ui.creator
     * @private
     */
    function clearOptionDetails() {
        detailsContainer.innerHTML = '';
    }

    /**
     * Sets current option name.
     *
     * @memberof app.ui.creator
     * @private
     */
    function setOption() {
        modelOption.setOption(
            optionsContainer.querySelector('.' + ACTIVE_CLASS).id
        );
    }

    /**
     * Updates indicator state.
     *
     * @memberof app.ui.creator
     * @private
     */
    function updateIndicator() {
        var all = indicatorContainer.querySelectorAll('.indicator-element'),
            active = indicatorContainer.querySelector('.' + ACTIVE_CLASS);

        if (active) {
            active.classList.remove(ACTIVE_CLASS);
        }
        all[activeOptionId].classList.add(ACTIVE_CLASS);
    }

    /**
     * Initializes option indicator elements.
     *
     * @memberof app.ui.creator
     * @private
     */
    function initializeIndicator() {
        var i = 0,
            length = options.length,
            indicatorElements = '';

        for (i = 0; i < length; i += 1) {
            indicatorElements += uiTemplates.getIndicatorElement();
        }

        indicatorContainer.innerHTML = indicatorElements;
        updateIndicator();
    }

    /**
     * Initializes options array storing references to the option HTML elements.
     *
     * @memberof app.ui.creator
     * @private
     */
    function initializeOptions() {
        options = [].slice.call(page.querySelectorAll(
            '.option.' + notificationType
        ));
    }

    /**
     * Resets slider id to zero and displays its first element.
     *
     * @memberof app.ui.creator
     * @private
     */
    function resetOptionSlider() {
        var i = 0,
            length = options.length;

        options[activeOptionId].classList.remove(ACTIVE_CLASS);
        activeOptionId = 0;
        options[activeOptionId].classList.add(ACTIVE_CLASS);

        for (i = 0; i < length; i += 1) {
            options[i].removeAttribute('style');
        }
    }

    /**
     * Resets parameters responsible for options appearance and behaviour.
     * Restores options to the initial state.
     *
     * @memberof app.ui.creator
     * @private
     */
    function resetOptions() {
        resetOptionSlider();
        modelOption.resetOption();
    }

    /**
     * Resets data related to the created notification.
     *
     * @memberof app.ui.creator
     * @private
     */
    function resetNotification() {
        modelNotification.clearData();
    }

    /**
     * Shows title and content option.
     * Notifies application that the title is incorrect.
     *
     * @memberof app.ui.creator
     * @private
     * @fires "title.incorrect"
     */
    function showAndHighlightTitle() {
        resetOptionSlider();
        updateIndicator();
        setOption();
        commonEvents.dispatchEvent('title.incorrect');
    }

    /**
     * Handles pageshow event.
     *
     * @memberof app.ui.creator
     * @private
     * @param {Event} ev
     */
    function onPageShow(ev) {
        if (doNotResetPageContent) {
            doNotResetPageContent = false;
            return;
        }

        notificationType = ev.detail.type;
        page.classList.add(notificationType);
        modelNotification.setType(notificationType);
        pageWidth = page.offsetWidth;
        initializeOptions();
        initializeIndicator();
        setOption();
    }

    /**
     * Handles pagehide event.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onPageHide() {
        if (!doNotResetPageContent) {
            page.classList.remove(notificationType);
            resetOptions();
            clearOptionDetails();
            resetNotification();
        }
    }

    /**
     * Handles click event on back button.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onBackBtnClick() {
        commonPages.changePage('main');
    }

    /**
     * Returns true when right option is available
     * for the given option id, false otherwise.
     *
     * @memberof app.ui.creator
     * @private
     * @param {number} id
     * @returns {boolean}
     */
    function isRightOptionAvailable(id) {
        return id < options.length - 1;
    }

    /**
     * Returns true when left option is available
     * for the given option id, false otherwise.
     *
     * @memberof app.ui.creator
     * @private
     * @param {number} id
     * @returns {boolean}
     */
    function isLeftOptionAvailable(id) {
        return id > 0;
    }

    /**
     * Handles touchstart event on options element.
     *
     * @memberof app.ui.creator
     * @private
     * @param {Event} ev
     */
    function onOptionsTouchStart(ev) {
        // save start position
        optionsXStart = ev.touches[0].clientX;
    }

    /**
     * Handles touchmove event on options element.
     *
     * @memberof app.ui.creator
     * @private
     * @param {Event} ev
     */
    function onOptionsTouchMove(ev) {
        var optionsXDelta = 0,
            optionsXVector = 0;

        // remove animation class if it is added
        optionsContainer.classList.remove(ANIMATED_CLASS);

        /**
         * calculate difference between start position and
         * position after current movement
         */
        optionsXDelta = ev.touches[0].clientX - optionsXStart;

        /**
         * calculate vector, which value is based on the current and previous
         * differences (between start position and position
         * after current movement)
         */
        optionsXVector = optionsXDelta - optionsXDeltaBuffer;

        /**
         * calculate sum of vectors, which is equal to total shift
         * of the option containers
         */
        optionsXVectorSum += optionsXVector;

        // modify left position of the active element using calculated value
        options[activeOptionId].style.left = optionsXVectorSum + 'px';

        /**
         * modify left position of the element located on the left side
         * of the active element
         */
        if (isLeftOptionAvailable(activeOptionId)) {
            options[activeOptionId - 1].style.left =
                options[activeOptionId - 1].offsetLeft + optionsXVector + 'px';
        }

        /**
         * modify left position of the element located on the right side
         * of the active element
         */
        if (isRightOptionAvailable(activeOptionId)) {
            options[activeOptionId + 1].style.left =
                options[activeOptionId + 1].offsetLeft + optionsXVector + 'px';
        }

        /**
         * save value of the difference between start position and
         * position after current movement for the next movement
         */
        optionsXDeltaBuffer = optionsXDelta;
    }

    /**
     * Handles touchend event on options element.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onOptionsTouchEnd() {
        /**
         * leftXPos, activeXPos and rightXPos are initialized with values that
         * correspond to the initial position (before movement) for left,
         * active and right elements respectively
         *
         * define greaterThanSwitchingThreshold variable which stores true
         * value when the total shift is greater than threshold calculated by
         * dividing device width value by switching threshold factor value,
         * false otherwise
         */
        var leftXPos = -pageWidth,
            activeXPos = 0,
            rightXPos = pageWidth,
            greaterThanSwitchingThreshold = Math.abs(optionsXVectorSum) >
                pageWidth / SWITCHING_THRESHOLD_FACTOR_CLASS;

        // if the total shift is different than zero, add animated class
        if (optionsXVectorSum !== 0) {
            optionsContainer.classList.add(ANIMATED_CLASS);
        }

        /**
         * modify leftXPos, activeXPos and rightXPos variables
         * when option elements can be moved right
         * and remove active class from active (center) element
         *
         * modified values are used later to update position
         * of the corresponding elements
         */
        if (greaterThanSwitchingThreshold && optionsXVectorSum > 0 &&
                isLeftOptionAvailable(activeOptionId)) {
            leftXPos = 0;
            activeXPos = pageWidth;
            rightXPos = pageWidth;
            options[activeOptionId].classList.remove(ACTIVE_CLASS);
        }

        /**
         * modify leftXPos, activeXPos and rightXPos variables
         * when option elements can be moved left
         * and remove active class from active (center) element
         *
         * modified values are used later to update position
         * of the corresponding elements
         */
        if (greaterThanSwitchingThreshold && optionsXVectorSum < 0 &&
                isRightOptionAvailable(activeOptionId)) {
            leftXPos = -pageWidth;
            activeXPos = -pageWidth;
            rightXPos = 0;
            options[activeOptionId].classList.remove(ACTIVE_CLASS);
        }

        /**
         * modify left position of the active element using activeXPos value
         *
         * active element is animated to this value thanks to the active class
         */
        options[activeOptionId].style.left = activeXPos + 'px';

        /**
         * when right element is available
         * modify left position of the right element using rightXPos value
         *
         * modify left position of the element located on the right side
         * of the right element (if it is available) using pageWidth value
         *
         * elements are animated to set values thanks to the active class
         *
         * when the position of the right element is set to zero,
         * it becomes active, so increment activeOptionId and
         * add active class to it
         */
        if (isRightOptionAvailable(activeOptionId)) {
            options[activeOptionId + 1].style.left = rightXPos + 'px';
            if (isRightOptionAvailable(activeOptionId + 1)) {
                options[activeOptionId + 2].style.left = pageWidth + 'px';
            }
            if (rightXPos === 0) {
                activeOptionId += 1;
                options[activeOptionId].classList.add(ACTIVE_CLASS);
                updateIndicator();
                setOption();
            }
        }

        /**
         * when left element is available
         * modify left position of the left element using leftXPos value
         *
         * modify left position of the element located on the left side
         * of the left element (if it is available) using -pageWidth value
         *
         * elements are animated to set values thanks to the active class
         *
         * when the position of the left element is set to zero,
         * it becomes active, so decrement activeOptionId and
         * add active class to it
         */
        if (isLeftOptionAvailable(activeOptionId)) {
            options[activeOptionId - 1].style.left = leftXPos + 'px';
            if (isLeftOptionAvailable(activeOptionId - 1)) {
                options[activeOptionId - 2].style.left = -pageWidth + 'px';
            }
            if (leftXPos === 0) {
                activeOptionId -= 1;
                options[activeOptionId].classList.add(ACTIVE_CLASS);
                updateIndicator();
                setOption();
            }
        }

        // reset variables used between touchstart and touchend events
        optionsXDeltaBuffer = 0;
        optionsXStart = 0;
        optionsXVectorSum = 0;
    }

    /**
     * Handles webkitTransitionEnd event on options element.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onOptionsTransitionEnd() {
        // remove active class when the transition is completed
        if (optionsContainer.classList.contains(ANIMATED_CLASS)) {
            optionsContainer.classList.remove(ANIMATED_CLASS);
        }
    }

    /**
     * Handles click event on code button.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onCodeBtnClick() {
        if (commonValidation.isReadyToSend()) {
            doNotResetPageContent = true;
            commonPages.changePage('code');
        } else {
            showAndHighlightTitle();
        }
    }

    /**
     * Handles click event on send button.
     *
     * @memberof app.ui.creator
     * @private
     */
    function onSendBtnClick() {
        if (commonValidation.isReadyToSend()) {
            modelNotification.sendNotification();
        } else {
            showAndHighlightTitle();
        }
    }

    /**
     * Handles softkeyboardchange event.
     *
     * @memberof app.ui.creator
     * @private
     * @param {Event} ev
     */
    function onSoftKeyboardChange(ev) {
        if (ev.state === 'on') {
            page.classList.add(KEYBOARD_ON_CLASS);
        } else {
            page.classList.remove(KEYBOARD_ON_CLASS);
        }
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator
     * @private
     */
    function bindEvents() {
        window.addEventListener('softkeyboardchange', onSoftKeyboardChange);
        page.addEventListener('pageshow', onPageShow);
        page.addEventListener('pagehide', onPageHide);
        backBtn.addEventListener('click', onBackBtnClick);
        optionsContainer.addEventListener('touchstart', onOptionsTouchStart);
        optionsContainer.addEventListener('touchmove', onOptionsTouchMove);
        optionsContainer.addEventListener('touchend', onOptionsTouchEnd);
        optionsContainer.addEventListener(
            'webkitTransitionEnd',
            onOptionsTransitionEnd
        );
        codeBtn.addEventListener('click', onCodeBtnClick);
        sendBtn.addEventListener('click', onSendBtnClick);
    }

    /**
     * Initializes the UI creator module.
     *
     * @memberof app.ui.creator
     * @public
     */
    function init() {
        page = document.getElementById('creator');
        backBtn = document.getElementById('back-btn');
        codeBtn = document.getElementById('code-btn');
        sendBtn = document.getElementById('send-btn');
        optionsContainer = page.querySelector('.options');
        indicatorContainer = page.querySelector('.indicator');
        detailsContainer = page.querySelector('.details');
        uiCreatorOption.init();
        bindEvents();
    }

    app.ui = app.ui || {};
    app.ui.creator = {
        init: init
    };

})(window.app);
