/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI creator option repeat module.
 * It is responsible for managing UI content,
 * displaying notification repeat option details.
 *
 * @module app.ui.creator.option.repeat
 * @requires {@link app.model.notification}
 * @requires {@link app.model.option}
 * @requires {@link app.ui.templates}
 * @namespace app.ui.creator.option.repeat
 * @memberof app.ui.creator.option
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiCreatorOptionRepeat(app) {
    'use strict';

    /**
     * UI templates module reference.
     *
     * @memberof app.ui.creator.option.repeat
     * @private
     * @type {object}
     */
    var uiTemplates = app.ui.templates,

        /**
         * Model notification module reference.
         *
         * @memberof app.ui.creator.option.repeat
         * @private
         * @type {object}
         */
        modelNotification = app.model.notification,

        /**
         * Parent container that contains option container.
         *
         * @memberof app.ui.creator.option.repeat
         * @private
         * @type {HTMLElement}
         */
        detailsContainer = null,

        /**
         * Increase button.
         *
         * @memberof app.ui.creator.option.repeat
         * @private
         * @type {HTMLElement}
         */
        increaseBtn = null,

        /**
         * Decrease button.
         *
         * @memberof app.ui.creator.option.repeat
         * @private
         * @type {HTMLElement}
         */
        decreaseBtn = null,

        /**
         * Repeat value element.
         *
         * @memberof app.ui.creator.option.repeat
         * @private
         * @type {HTMLElement}
         */
        repeatValue = null;

    /**
     * Updates displayed repeat value.
     *
     * @memberof app.ui.creator.option.repeat
     * @private
     * @param {number} value
     */
    function updateValue(value) {
        repeatValue.innerText = value;
    }

    /**
     * Handles click event on increase button.
     *
     * @memberof app.ui.creator.option.repeat
     * @private
     */
    function onIncreaseBtnClick() {
        modelNotification.setRepeat(1, updateValue);
    }

    /**
     * Handles click event on decrease button.
     *
     * @memberof app.ui.creator.option.repeat
     * @private
     */
    function onDecreaseBtnClick() {
        modelNotification.setRepeat(-1, updateValue);
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.creator.option.repeat
     * @private
     */
    function bindEvents() {
        increaseBtn.addEventListener('click', onIncreaseBtnClick);
        decreaseBtn.addEventListener('click',  onDecreaseBtnClick);
    }

    /**
     * Shows repeat option content.
     *
     * @memberof app.ui.creator.option.repeat
     * @public
     */
    function show() {
        detailsContainer.innerHTML = uiTemplates.getRepeatOptionContent({
            repeat: modelNotification.getRepeat()
        });
        increaseBtn = document.querySelector('.repeat-option .increase-btn');
        decreaseBtn = document.querySelector('.repeat-option .decrease-btn');
        repeatValue = document.querySelector('.repeat-option .value');
        bindEvents();
    }

    /**
     * Initializes the UI creator option repeat module.
     *
     * @memberof app.ui.creator.option.repeat
     * @public
     */
    function init() {
        detailsContainer = document.querySelector('.details');
    }

    app.ui = app.ui || {};
    app.ui.creator = app.ui.creator || {};
    app.ui.creator.option = app.ui.creator.option || {};
    app.ui.creator.option.repeat = {
        init: init,
        show: show
    };

})(window.app);
