/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, document*/

/**
 * Application UI main module.
 * It is responsible for managing user interface.
 *
 * @module app.ui.main
 * @requires {@link app.common.pages}
 * @namespace app.ui.main
 * @memberof app.ui
 */

// make sure that "app" namespace is created
window.app = window.app || {};

(function defineAppUiMain(app) {
    'use strict';

    /**
     * Common pages module reference.
     *
     * @memberof app.ui.main
     * @private
     * @type {object}
     */
    var commonPages = app.common.pages,

        /**
         * Page element.
         *
         * @memberof app.ui.main
         * @private
         * @type {HTMLElement}
         */
        page = null,

        /**
         * Simple button.
         *
         * @memberof app.ui.main
         * @private
         * @type {HTMLElement}
         */
        simpleBtn = null,

        /**
         * Ongoing button.
         *
         * @memberof app.ui.main
         * @private
         * @type {HTMLElement}
         */
        ongoingBtn = null,

        /**
         * Progress button.
         *
         * @memberof app.ui.main
         * @private
         * @type {HTMLElement}
         */
        progressBtn = null;

    /**
     * Handles click event on simple button.
     * Makes the creator page active.
     *
     * @memberof app.ui.main
     * @private
     */
    function onSimpleBtnClick() {
        commonPages.changePage('creator', {type: 'simple'});
    }

    /**
     * Handles click event on ongoing button.
     * Makes the creator page active.
     *
     * @memberof app.ui.main
     * @private
     */
    function onOngoingBtnClick() {
        commonPages.changePage('creator', {type: 'ongoing'});
    }

    /**
     * Handles click event on progress button.
     * Makes the creator page active.
     *
     * @memberof app.ui.main
     * @private
     */
    function onProgressBtnClick() {
        commonPages.changePage('creator', {type: 'progress'});
    }

    /**
     * Handles pageshow event.
     *
     * @memberof app.ui.main
     * @private
     */
    function onPageShow() {
        return;
    }

    /**
     * Handles pagehide event.
     *
     * @memberof app.ui.main
     * @private
     */
    function onPageHide() {
        return;
    }

    /**
     * Registers event listeners.
     *
     * @memberof app.ui.main
     * @private
     */
    function bindEvents() {
        simpleBtn.addEventListener('click', onSimpleBtnClick);
        ongoingBtn.addEventListener('click', onOngoingBtnClick);
        progressBtn.addEventListener('click', onProgressBtnClick);
        page.addEventListener('pageshow', onPageShow);
        page.addEventListener('pagehide', onPageHide);
    }

    /**
     * Initializes the UI main module.
     *
     * @memberof app.ui.main
     * @public
     */
    function init() {
        page = document.getElementById('main');
        simpleBtn = page.querySelector('#simple-btn');
        ongoingBtn = page.querySelector('#ongoing-btn');
        progressBtn = page.querySelector('#progress-btn');

        bindEvents();
    }

    app.ui = app.ui || {};
    app.ui.main = {
        init: init
    };

})(window.app);
